  
  import Foundation
  import Flutter
  
  @UIApplicationMain
  @objc class AppDelegate: FlutterAppDelegate, TapResearchSurveyDelegate, TapResearchRewardDelegate {
    var tapresearchPlacement = TRPlacement()
    var channel: FlutterMethodChannel!
    
    public func tapResearchSurveyWallOpened(with placement: TRPlacement) {
        channel.invokeMethod("tapResearchSurveyWallOpened", arguments: nil)
    }
    
    public func tapResearchSurveyWallDismissed(with placement: TRPlacement) {
        channel.invokeMethod("tapResearchSurveyWallDismissed", arguments: nil)
    }
    
    public func tapResearchDidReceive(_ reward: TRReward) {
        channel.invokeMethod("tapResearchDidReceiveReward", arguments: reward.rewardAmount)
    }
    
    override func application(
        _ application: UIApplication,
        didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?
    ) -> Bool {
        let rootViewController : FlutterViewController = window?.rootViewController as! FlutterViewController
        channel = FlutterMethodChannel(name: "tapr", binaryMessenger: rootViewController as! FlutterBinaryMessenger)
        channel.setMethodCallHandler {(call: FlutterMethodCall, result: FlutterResult) -> Void in
            switch call.method {
            case "init":
                if let args = call.arguments as? Dictionary<String, Any>,
                    let userId = args["user_id"] as? String, let apiToken = args["api_token"] as? String{
                    TapResearch.setUniqueUserIdentifier(userId)
                    TapResearch.initWithApiToken(apiToken, delegate: self)
                    result(nil)
                } else {
                    result(FlutterError.init(code: "errorInit", message: "Couldn't initialize TapResearch", details: nil))
                }
            case "initPlacement":
                if let args = call.arguments as? Dictionary<String, Any>,
                    let placementId = args["placement_id"] as? String {
                    TapResearch.initPlacement(withIdentifier: placementId, placementBlock: { placement in
                        self.tapresearchPlacement = placement
                        if self.tapresearchPlacement.placementCode != PLACEMENT_CODE_SDK_NOT_READY {
                            if self.tapresearchPlacement.isSurveyWallAvailable == true {
                                self.channel.invokeMethod("isSurveyWallAvailable", arguments: 1)
                            }
                        } else {
                            self.channel.invokeMethod("isSurveyWallAvailable", arguments: 0)
                        }
                    })
                } else {
                    result(FlutterError.init(code: "errorInit", message: "Couldn't initialize TapResearch", details: nil))
                }
                
            case "showSurveyWall":
                self.tapresearchPlacement.showSurveyWall(with: nil)
            case "setNavigationBarText":
                let navBarText = call.arguments as? String
                TapResearch.setNavigationBarText(navBarText!)
            case "setNavigationBarColor":
                let navBarColor = call.arguments as? String
                let color =  hexStringToUIColor(hex: navBarColor!)
                TapResearch.setNavigationBarColor(color)
            case "setNavigationBarTextColor":
                let navBarTextColor = call.arguments as? String
                let color =  hexStringToUIColor(hex: navBarTextColor!)
                TapResearch.setNavigationBarTextColor(color)
            default: result(FlutterMethodNotImplemented)
            GeneratedPluginRegistrant.register(with: self)
            }
        }
        return super.application(application, didFinishLaunchingWithOptions: launchOptions)
    }
  }
  
  func hexStringToUIColor (hex:String) -> UIColor {
    var cString:String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()
    
    if (cString.hasPrefix("#")) {
        cString.remove(at: cString.startIndex)
    }
    
    if ((cString.count) != 6) {
        return UIColor.gray
    }
    
    var rgbValue:UInt64 = 0
    Scanner(string: cString).scanHexInt64(&rgbValue)
    
    return UIColor(
        red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
        green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
        blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
        alpha: CGFloat(1.0)
    )
  }
